<?php

namespace ForteEcom\Payments\Block;

use Exception;
use ForteEcom\Payments\Model\Payment\ForteEcomHelper;
use ForteEcom\Payments\Model\Ui\ConfigProvider;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\Http\Context;
use Magento\Sales\Api\Data\OrderPaymentInterface;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface;
use Magento\Vault\Api\PaymentTokenManagementInterface;

class Success extends \Magento\Checkout\Block\Onepage\Success
{
    public $_forteecom_order_id;
    public $_forteecom_order_status;
    public $_forteecom_session_id;
    public $_forteecom_merchant_id;
    public $order_description;
    public $currency;
    public $order_id;
    public $amount;
    public $return_url;
    public $notify_url;
    public $config_provider;
    public $card_save_url;
    public $init_payment_url;
    public $customer_id;
    public $save_card_state;
    public $script;
    protected $transaction_builder;
    protected $token_management;
    protected $_template = 'ForteEcom_Payments::order/success.phtml';
    /** @var OrderPaymentInterface */
    private $payment;
    /** @var Order */
    private $order;

    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        Session $checkoutSession,
        PaymentTokenManagementInterface $token_management,
        BuilderInterface $transaction_builder,
        Order\Config $orderConfig,
        Context $httpContext,
        ConfigProvider $config_provider,
        array $data = []
    )
    {
        parent::__construct($context, $checkoutSession, $orderConfig, $httpContext, $data);
        $this->config_provider     = $config_provider;
        $this->transaction_builder = $transaction_builder;
        $this->token_management    = $token_management;
    }

    /**
     * Prepare order data and process status if vault used
     *
     * @throws Exception
     */
    public function process_order_data()
    {
        $this->order   = $this->_checkoutSession->getLastRealOrder();
        $this->payment = $this->order->getPayment();

        if ($this->is_forteecom_payment() && $this->is_vault_used()) {
            $this->token_management->addLinkToOrderPayment($this->payment->getAdditionalInformation()['payment_token_id'], $this->payment->getEntityId());
            ForteEcomHelper::update_payment_status($this->order, $this->payment, $this->payment->getAdditionalInformation()['_forteecom_order_status'], ForteEcomHelper::STATUS_INIT, $this->transaction_builder);
        }
    }

    /**
     *  Checks if order placed using ForteEcom
     *
     * @return bool
     */
    public function is_forteecom_payment()
    {
        return ($this->payment->getMethod() == 'forteecom');
    }

    /**
     *  Checks if vault vault was used during checkout
     *
     * @return string
     */
    public function is_vault_used()
    {
        return $this->payment->getAdditionalInformation()['is_vault_used'];
    }

    /**
     * Checks if forteecom widget button needs to be shown
     *
     * @return bool
     */
    public function needToShowForteEcomPayment()
    {
        return ($this->is_forteecom_payment() && !$this->is_vault_used());
    }

    /**
     * Prepares ForteEcom init data
     */
    public function prepareForteEcomData()
    {
        $order_id          = $this->order->getId();
        $order_description = sprintf('%s - Order: %s', $this->order->getStoreName(), $this->order->getIncrementId());
        $this->amount      = $this->order->getGrandTotal();
        $config            = $this->config_provider->getConfig();

        $this->_forteecom_merchant_id = $config['merchant_id'];

        $init_result = ForteEcomHelper::init_payment($this->_forteecom_merchant_id, $config['pre_auth'], $this->amount, $order_id, $order_description, 'KZT');

        $this->_forteecom_order_id   = $init_result['_forteecom_order_id'];
        $this->_forteecom_session_id = $init_result['_forteecom_session_id'];
        $this->order_description     = $order_description;
        $this->currency              = 398;
        $this->order_id              = $order_id;
        $this->notify_url            = $this->_urlBuilder->getUrl('forteecom/response/processstatus');
        $this->return_url            = $this->_urlBuilder->getUrl('forteecom/response/index');
        $this->card_save_url         = $this->_urlBuilder->getUrl('forteecom/response/addcard');
        $this->init_payment_url      = $this->_urlBuilder->getUrl('forteecom/response/initpayment');
        $this->customer_id           = $this->order->getCustomerId();
        $this->save_card_state       = (empty($this->customer_id)) ? 'hidden' : 'unchecked';
        $this->script                = ForteEcomHelper::get_script();

        $this->payment->setAdditionalInformation('_forteecom_order_id', $this->_forteecom_order_id);
        $this->payment->setAdditionalInformation('_forteecom_order_status', ForteEcomHelper::STATUS_INIT);
        $this->payment->setAdditionalInformation('_forteecom_session_id', $this->_forteecom_session_id);
        $this->payment->setAdditionalInformation('_forteecom_merchant_id', $this->_forteecom_merchant_id);

        $this->order->setState(Order::STATE_PENDING_PAYMENT);
        $this->order->addCommentToStatusHistory('ForteEcom: State changed to ' . Order::STATE_PENDING_PAYMENT, true);
        $this->order->save();
        $this->payment->save();

    }
}
