<?php
declare(strict_types=1);

namespace ForteEcom\Payments\Controller\Response;

use Exception;
use ForteEcom\Payments\Model\Payment\ForteEcomHelper;
use ForteEcom\Payments\Model\Ui\ConfigProvider;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Action\HttpGetActionInterface;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Message\ManagerInterface;
use Magento\Framework\View\Result\PageFactory;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface;

/**
 * implementing get, post and csrf interfaces so that it can handle requests from outside
 *
 * @link https://devdocs.magento.com/guides/v2.3/extension-dev-guide/routing.html
 *
 * Class Index
 */
class Index extends Action implements HttpGetActionInterface, HttpPostActionInterface, CsrfAwareActionInterface
{
    /**
     * @var PageFactory
     */
    protected $page_factory;
    protected $transaction_builder;
    protected $config_provider;

    /**
     * @param Context          $context
     * @param PageFactory      $pageFactory
     * @param BuilderInterface $transactionBuilder
     * @param ManagerInterface $messageManager
     */
    public function __construct(
        Context $context,
        ConfigProvider $config_provider,
        PageFactory $pageFactory,
        BuilderInterface $transactionBuilder,
        ManagerInterface $messageManager
    )
    {
        parent::__construct($context);

        $this->config_provider     = $config_provider;
        $this->transaction_builder = $transactionBuilder;
        $this->page_factory        = $pageFactory;
    }

    /**
     * @return ResponseInterface|ResultInterface|void
     * @throws Exception
     */
    public function execute()
    {

        $order_id = $this->getRequest()->getParam('order_id');

        /** @var Order $order */
        $order   = ObjectManager::getInstance()->create('\Magento\Sales\Model\Order')->load($order_id);
        $payment = $order->getPayment();
        $this->config_provider->getConfig();
        $current_data  = $payment->getAdditionalInformation();
        $status_result = ForteEcomHelper::get_payment_status($current_data['_forteecom_merchant_id'], $current_data['_forteecom_order_id'], $current_data['_forteecom_session_id']);
        if (empty($status_result['error_message'])) {
            ForteEcomHelper::update_payment_status($order, $payment, $status_result['_forteecom_order_status'], $current_data['_forteecom_order_status'], $this->transaction_builder);
            ForteEcomHelper::process_payment_status($status_result['_forteecom_order_status'], $this->messageManager);
        } else {
            $this->messageManager->addErrorMessage(__('Error occurred while getting status for order'));
            $order->addCommentToStatusHistory('Error occurred while getting status for order #' . $order->getId());
        }

        $this->_redirect('sales/order/history');
    }

    /**
     * @inheritDoc
     */
    public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException
    {
        return null;
    }

    /**
     * @inheritDoc
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }
}
