<?php
declare(strict_types=1);

namespace ForteEcom\Payments\Controller\Response;

use ForteEcom\Payments\Model\Payment\ForteEcomHelper;
use ForteEcom\Payments\Model\Ui\ConfigProvider;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\PaymentException;
use Magento\Framework\View\Result\PageFactory;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface;

/**
 * implementing post and csrf interfaces so that it can handle requests from outside
 *
 * @link https://devdocs.magento.com/guides/v2.3/extension-dev-guide/routing.html
 *
 * Class Index
 */
class ProcessStatus extends Action implements HttpPostActionInterface, CsrfAwareActionInterface
{
    /**
     * @var PageFactory
     */
    protected $page_factory;
    protected $json_factory;
    protected $transaction_builder;
    protected $config_provider;

    /**
     * @param Context          $context
     * @param PageFactory      $pageFactory
     * @param JsonFactory      $jsonFactory
     * @param BuilderInterface $transactionBuilder
     */
    public function __construct(
        Context $context,
        ConfigProvider $config_provider,
        PageFactory $pageFactory,
        JsonFactory $jsonFactory,
        BuilderInterface $transactionBuilder
    )
    {
        parent::__construct($context);

        $this->page_factory        = $pageFactory;
        $this->config_provider     = $config_provider;
        $this->json_factory        = $jsonFactory;
        $this->transaction_builder = $transactionBuilder;
    }

    /**
     * @return ResponseInterface|Json|ResultInterface
     * @throws PaymentException
     */
    public function execute()
    {
        $request  = json_decode($this->getRequest()->getContent(), true);
        $order_id = $request['data']['remoteId'];

        /** @var Order $order */
        $order   = ObjectManager::getInstance()->create('\Magento\Sales\Model\Order')->load($order_id);
        $payment = $order->getPayment();

        $current_data  = $payment->getAdditionalInformation();
        $this->config_provider->getConfig();
        $status_result = ForteEcomHelper::get_payment_status($current_data['_forteecom_merchant_id'], $current_data['_forteecom_order_id'], $current_data['_forteecom_session_id']);
        if (empty($status_result['error_message'])) {
            ForteEcomHelper::update_payment_status($order, $payment, $status_result['_forteecom_order_status'], $current_data['_forteecom_order_status'], $this->transaction_builder);
        } else {
            $order->addCommentToStatusHistory('Error occurred while getting status for order #' . $order->getId());
        }

        return $this->json_factory->create()->setData(['success' => 1]);
    }

    /**
     * @inheritDoc
     */
    public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException
    {
        return null;
    }

    /**
     * @inheritDoc
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }
}
