define([
    'jquery',
    'Magento_Vault/js/view/payment/method-renderer/vault',
    'Magento_Ui/js/model/messageList',
    'Magento_Checkout/js/action/place-order'
], function ($, VaultComponent, globalMessageList, placeOrderAction) {
    'use strict';

    return VaultComponent.extend({
        defaults: {
            template: 'ForteEcom_Payments/payment/checkout_vault'
        },

        /**
         * Get card mask
         * @returns {*}
         */
        getMaskedCard: function () {
            return this.details.last4;
        },

        /**
         * Get expiration date
         * @returns {String}
         */
        getExpirationDate: function () {
            return this.details.expiration_date;
        },

        /**
         * Get card type
         * @returns {String}
         */
        getCardType: function () {
            return this.details.type;
        },

        /**
         * Get card token
         * @returns {String}
         */
        getToken: function () {
            return this.details.public_hash;
        },

        /**
         * Get card CVV
         * @returns {*|jQuery}
         */
        getCVV: function () {
            return $('#' + this.getId() + '_cvv').val();
        },

        /**
         * Add fail handler to show Exception message which was thrown during placeOrder
         * @returns {jQuery}
         */
        getPlaceOrderDeferredObject: function () {
            return $.when(
                placeOrderAction(this.getData(), this.messageContainer)
                    .fail(
                        function (response) {
                            let error = JSON.parse(response.responseText);
                            globalMessageList.addErrorMessage({
                                message: error.message
                            });
                        }
                    )
            );
        },

        /**
         * Get card additional data for orderPlace
         * @returns {{additional_data: {cvv: (*|jQuery), public_hash: String}, method: *}}
         */
        getData: function () {
            return {
                method: this.getCode(),
                additional_data: {
                    public_hash: this.getToken(),
                    cvv: this.getCVV()
                }
            };
        }
    });
});
